/*
dtmflogd        A program to log digits captured by a serial dtmf decoder
Copyright (C) 2003  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/poll.h>
#include <errno.h>

#include "logging.h"
#include "pnum.h"
#include "area_codes.h"

char mustrun=1;
FILE *log_fptr=NULL;

static int log_number(char *number);
static int log_message(char *message);
static int log_function(char *func_str, char *func_name);
void get_timeStamp(char *buf);

int init_logging(char *log_file)
{
	log_fptr = fopen(log_file, "a+");
	if (log_fptr==NULL) { 
		perror("Could not open log file");
		return -1; 
	}

	return 0;
}

#define LOG_STATE_WAIT_FIRST 1
#define LOG_STATE_WAIT_NEXTDIGIT 2
int start_logging(int fd)
{
	int res;
	unsigned char digit;
	unsigned char state;
	struct pollfd pfd;
	
	char number[NUMBER_MAX_LEN+1];
	int numbers=0;

	memset(number, 0, NUMBER_MAX_LEN+1);
	
	/* init_logging must be called first */
	if (log_fptr==NULL) { return -1; }

	/* Prepare the poll_fd */
	pfd.fd = fd;
	pfd.events = POLLIN;
	
	log_message("dtmfd logging started");
	state = LOG_STATE_WAIT_FIRST;
	while(mustrun)
	{
		res = poll(&pfd, 1, TIME_FOR_NUMBER);
				
		if (res==0) // timeout
		{
			if (state==LOG_STATE_WAIT_NEXTDIGIT)
			{
				log_number(number);
				/* reset the buffer to 0 */
				numbers=0;
				memset(number, 0, NUMBER_MAX_LEN-1);
				state=LOG_STATE_WAIT_FIRST;
			}
		} else if (res>0) { // incoming data
			res = read(fd, &digit, 1);
#ifdef TRANSLATE
			if (digit==0xFF) digit='7'; // mystery
#endif
			state=LOG_STATE_WAIT_NEXTDIGIT;
			if (res==1) {
				if (numbers>=NUMBER_MAX_LEN-1) {
					log_message("NUMBER_MAX_LEN-1 exceeded. Flushing buffer...");
					log_number(number);
					/* reset the buffer to 0 */
					numbers=0;
					memset(number, 0, NUMBER_MAX_LEN-1);
				}
				number[numbers++]=digit;
			}
		} else {
			if (errno!=EINTR) {
				log_message("poll error");
			} else {
				log_message("Received signal to exit");
			}
		}
	}
	
	if (state==LOG_STATE_WAIT_NEXTDIGIT)
	{
		log_number(number);
	}
	
	log_message("dtmfd logging stopped");
	
	return 0;
}

void shutdown_logging(void)
{
	if (log_fptr!=NULL)
	{
		fclose(log_fptr);
	}
}

static int log_number(char *number)
{
	PhoneNumber pnum;
	int res;
	char timestamp[40];
	char phone_number_without_function[PNUM_MAX_LEN];
	char phone_function[PNUM_MAX_LEN];
	char pretty_number[PNUM_MAX_LEN];
	char *city_name=NULL;
	char *company_name=NULL;
	AreaInfo area_info;
	
	get_timeStamp(timestamp);

	/* init the strings with blanks */
	memset(phone_number_without_function, 0, PNUM_MAX_LEN);
	memset(pretty_number, 0, PNUM_MAX_LEN);
	memset(phone_function, 0, PNUM_MAX_LEN);
	phone_number_without_function[0]=0;
	phone_function[0]=0;
	pretty_number[0]=0;
	
	res = pnum_remove_function(number, 
					phone_function, 
						phone_number_without_function);
	if (res>=0)
	{
		if (res>0) // there was a function
		{
			log_function(phone_function,
					pnum_getFunctionName(phone_function));
		}
		
		res = pnum_split_and_expand(phone_number_without_function, &pnum);
		if (res<0)
		{
			strncpy(pretty_number, "Nan", PNUM_MAX_LEN);
		}
		else 
		{
			pnum_toString(pretty_number, &pnum);

			res = getAreaInfo(pnum.npa, pnum.co, &area_info);
			if (res>=0)
			{
				company_name=area_info.company;
				city_name=area_info.rate_center;	
			}
		}
	}
	else {
		/* Failed to extract and/or recognize a phone number
		 * and/or a * function */
		strncpy(pretty_number, "Nan", PNUM_MAX_LEN);
	}

	fprintf(log_fptr, "# %s , %s , %s", timestamp, number, pretty_number);
	if (company_name!=NULL&&city_name!=NULL) {
		fprintf(log_fptr, " , %s , %s", city_name, company_name);
	}
	fprintf(log_fptr, "\n");
	fflush(log_fptr);
	
	return 0;
}

static int log_message(char *message)
{
	char timestamp[40];

	get_timeStamp(timestamp);
	
	fprintf(log_fptr, "? %s , %s\n", timestamp, message);
	fflush(log_fptr);

	return 0;
}

static int log_function(char *func_str, char *func_name)
{
	char timestamp[40];

	get_timeStamp(timestamp);
	
	fprintf(log_fptr, "F %s , %s , %s\n", timestamp, func_str, func_name);
	fflush(log_fptr);

	return 0;
}

void get_timeStamp(char *buf)
{
	time_t now;
	char *timestr=NULL;
	char *tmp;
	
	/* get the current time and convert it
	 * to a string */
	now = time(NULL);
	timestr = strdup(ctime(&now));	

	/* Copy the string in the buffer */
	sprintf(buf, "%s", timestr);
	
	/* Remove the trailing \n */
	tmp = strstr(buf, "\n");
	if (tmp!=NULL) {
		tmp[0] = 0;
	}

	return;
}


