/*
dtmflogd        A program to log digits captured by a serial dtmf decoder
Copyright (C) 2003  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>


/** Purpose: parse a string representing the number of data bits, parity, and 
 * the number of stop bits used for serial communication.
 * eg: 8N1 7E1 7N1 etc...
 *
 * returns a negative value if there are missing arguments,
 * too many arguments, or invalid ones. 
 *
 * If the string is valid, the corresponding bits will 
 * be or'd with or_bits. 
 *
 * NOTE: the or_bits value is intended to be used with the
 * c_cflag memeber of the struct termios. You can find additional information
 * by reading the Serial Programming Guide for POSIX Operating Systems.
 * http://www.easysw.com/~mike/serial/serial.html
 * */
int getModeBits(char *modestr, unsigned long *or_bits)
{
	unsigned long tmp_bits=0;
		
	if (strlen(modestr)!=3) { return -1; }

	switch(modestr[0])
	{
		case '8': tmp_bits |= CS8; break;
		case '7': tmp_bits |= CS7; break;
		case '6': tmp_bits |= CS6; break;
		case '5': tmp_bits |= CS5; break;
		default: return -1;
	}	
	switch(modestr[1])
	{
		case 'N': tmp_bits |= 0; break;
		case 'O': tmp_bits |= PARENB|PARODD; break;
		case 'E': tmp_bits |= PARENB; break;
		default: return -1;
	}
	switch(modestr[2])
	{ 
	 	case '1': tmp_bits |= 0; break;
		case '2': tmp_bits |= CSTOPB; break;
		default: return -1;
	}
	
	*or_bits |= tmp_bits;	
	return 0;
}


/** Purpose: Convert an integer representing a baud rate 
 * to bits.
 *
 * returns a value with some bit(s) set according to the
 * baud rate specified by rate.
 * If 0 is returned, then the specified baud rate is invalid/unknown.
 * 
 *
 * NOTE: the returned bits value is intended to be used with the
 * c_cflag memeber of the struct termios. You can find additional information
 * by reading the Serial Programming Guide for POSIX Operating Systems.
 * http://www.easysw.com/~mike/serial/serial.html
 * */
unsigned long getBaudRateBits(int rate)
{
	switch(rate)
	{
		case 1200: return B1200; break;
		case 2400: return B2400; break;		
		case 4800: return B4800; break;
		case 9600: return B9600; break;
		case 19200: return B19200; break;
		case 38400: return B38400; break;		       
		case 57600: return B57600; break;
		case 115200: return B115200; break;
		case 230400: return B230400; break;
		case 460800: return B460800; break;
		case 500000: return B500000; break;
		case 576000: return B576000; break;
		default: return 0;
	}
	return 0;
}

int openport(char *file, struct termios *oldtio, unsigned long c_cflag)
{
	int fd;
	struct termios newtio;

	printf("Opening device %s...", file); fflush(stdout);
	fd = open(file, O_RDWR | O_NOCTTY | O_NONBLOCK);

	fcntl(fd, F_SETFL, 0);
	
	if (fd<0) { perror(file); return -1; }
	printf("done\n");

	tcgetattr(fd, oldtio);

	memset(&newtio,0,sizeof(newtio));

	newtio.c_cflag = c_cflag | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VTIME] = 0;
	newtio.c_cc[VMIN] = 1; // block until 1 char received
	
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio);

	/* raise dtr */
//	status = TIOCM_DTR;
//	ioctl(fd, TIOCMSET, &status);

	
	
	return fd;
}


